#-*- coding: utf-8 -*-


# Python Imports
import decimal
import hashlib
import ipaddress
import inspect
import logging
import os
import pprint
import random
import re
import string
import xml


def absdir(path):
    return os.path.abspath(os.path.dirname(path))

def get_caller():
    f = inspect.currentframe()
    f = f.f_back
    f = f.f_back
    filename = f.f_code.co_filename
    module = filename.split('/')[-1].split('.')[0]
    lineno = f.f_lineno
    return "%s:%s" % (module, lineno)

def logw(var, msg=''):
    caller = get_caller()
    logging.warning('%s %s %s %s', cstr(caller, 'yellow'), msg, type(var), pprint.pformat(var))

def logline(*args):
    caller = get_caller()
    msg = '%s %s' % (cstr(caller, 'yellow').ljust(20), args[0])
    logging.warning(msg, *args[1:])

def force_int(string):
    if type(string) in (int, int): return int(string)
    int_re = re.compile('^.*?(\d+).*?$')
    match = int_re.match(string)
    if not match: return None
    return int(match.group(1))

def md5hex(val):
    return hashlib.md5(val).hexdigest()

def html_unescape(val):
    return xml.sax.saxutils.unescape(val, {'&quot;': '"'})

def split_name(name):
    if not name or type(name) is not str:
        return None
    names = name.split()
    if len(names) == 1:
        return {'first_name': name, 'last_name': ''}
    else:
        first = ' '.join(name.split()[:-1])
        last = name.split()[-1]
        return {'first_name': first, 'last_name': last}

def map_items(items, key):
    return dict([(item[key], item) for item in items])


# Make it easier to use ansi escape sequences for terminal colors
colors = {'black' : 30, 'red' : 31, 'green' : 32, 'yellow' : 33, 'blue' : 34,
          'magenta' : 35, 'cyan' : 36, 'white' : 37, 'reset': 39, }
attrs = {'reset':'0', 'bold':'1', 'faint':'2', 'regular':'2',
         'underscore':'4', 'blink':'5', 'reverse':'7'}

def ansi_esc(colorName, **kwargs):
    out = '\x1B['
    if 'attr' in kwargs and kwargs['attr'] in attrs:
        out += attrs[kwargs['attr']] + ';'
    if 'bgcolor' in kwargs and kwargs['bgcolor'] in colors:
        bgcolor = colors[kwargs['bgcolor']] + 10
        out += str(bgcolor) + ';'
    out += str(colors[colorName]) + 'm'
    return out

def cline(line, mode):
    pick_ansi = {'-': 'red', '+': 'green', '*': 'green', '@': 'cyan'}
    color = pick_ansi.get(line[0], '')
    if color != '':
        line = ansi_esc(color) + line + ansi_esc('reset')
    if mode == 'print': print(line)
    if mode == 'return': return line

def cdiff(line):
    pick_ansi = {'-': 'red', '+': 'green', '*': 'green'}
    color = pick_ansi.get(line[0], '')
    if color != '':
        return ansi_esc(color) + line + ansi_esc('reset')
    else:
        return line

def cstr(data, color):
    return ansi_esc(color) + data + ansi_esc('reset')


def magic_key(length=8):
    return reduce(lambda x, y: x + random.choice(string.ascii_letters + string.digits), range(length), '').lower()


def validate_int(value):
    # int() can't take a None so we have to check that first
    if value is None:
        return None
    # As long as it's a string we can remove commas, since those don't validate
    if type(value) is str:
        value = value.replace(',', '')
    try:
        return int(value)
    except ValueError:
        return None

def validate_date(value):
    if not value:
        return None
    if type(value) is datetime.datetime:
        return value
    try:
        return dateutil.parser.parse(value)
    except ValueError:
        return None

def validate_bool(value):
    if valid is None:
        return None
    if type(value) is bool:
        return value
    return bool(value)

def validate_decimal(value):
    if value is None:
        return None
    try:
        value = decimal.Decimal(re.sub(r'[^\d.]', '', value))
    except decimal.InvalidOperation:
        return None
    return value

def validate_email(value):
    try:
        logging.error("IT IS TIME. Port the Django from below")
        cleaned_email = django.forms.EmailField(required=True).clean(value)
        return cleaned_email
    except django.forms.ValidationError:
        logging.warning("Invalid email: %s", value)
        return None

def validate_ip_address(value):
    try:
        ip = ipaddress.ip_address(value)
        return ip
    except:
        return None



### From Django: https://github.com/django/django/blob/11b8c30b9e02ef6ecb996ad3280979dfeab700fa/django/core/validators.py

#@deconstructible
#class EmailValidator:
#    message = _('Enter a valid email address.')
#    code = 'invalid'
#    user_regex = _lazy_re_compile(
#        r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*\Z"  # dot-atom
#        r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*"\Z)',  # quoted-string
#        re.IGNORECASE)
#    domain_regex = _lazy_re_compile(
#        # max length for domain name labels is 63 characters per RFC 1034
#        r'((?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+)(?:[A-Z0-9-]{2,63}(?<!-))\Z',
#        re.IGNORECASE)
#    literal_regex = _lazy_re_compile(
#        # literal form, ipv4 or ipv6 address (SMTP 4.1.3)
#        r'\[([A-f0-9:\.]+)\]\Z',
#        re.IGNORECASE)
#    domain_whitelist = ['localhost']
#
#    def __init__(self, message=None, code=None, whitelist=None):
#        if message is not None:
#            self.message = message
#        if code is not None:
#            self.code = code
#        if whitelist is not None:
#            self.domain_whitelist = whitelist
#
#    def __call__(self, value):
#        if not value or '@' not in value:
#            raise ValidationError(self.message, code=self.code)
#
#        user_part, domain_part = value.rsplit('@', 1)
#
#        if not self.user_regex.match(user_part):
#            raise ValidationError(self.message, code=self.code)
#
#        if (domain_part not in self.domain_whitelist and
#                not self.validate_domain_part(domain_part)):
#            # Try for possible IDN domain-part
#            try:
#                domain_part = domain_part.encode('idna').decode('ascii')
#            except UnicodeError:
#                pass
#            else:
#                if self.validate_domain_part(domain_part):
#                    return
#            raise ValidationError(self.message, code=self.code)
#
#    def validate_domain_part(self, domain_part):
#        if self.domain_regex.match(domain_part):
#            return True
#
#        literal_match = self.literal_regex.match(domain_part)
#        if literal_match:
#            ip_address = literal_match.group(1)
#            try:
#                validate_ipv46_address(ip_address)
#                return True
#            except ValidationError:
#                pass
#        return False
#
#    def __eq__(self, other):
#        return (
#            isinstance(other, EmailValidator) and
#            (self.domain_whitelist == other.domain_whitelist) and
#            (self.message == other.message) and
#            (self.code == other.code)
#        )



#!/usr/bin/env python3
#-*- coding: utf-8 -*-
#
# Aegis is your shield to protect you on the Brave New Web

import argparse
import logging
import os
import stdlib
import sys

parser = argparse.ArgumentParser(description='Create your shield.')
parser.add_argument('cmd', metavar='<command>', type=str, nargs=1, help='What to do: [create]')
parser.add_argument('--appname', metavar='<appname>', type=str, nargs=1, help='Name for the forked copy')
parser.add_argument('--domain', metavar='<domain>', type=str, nargs=1, help='Domain for the forked copy')
#parser.add_argument('values', metavar='<value>', type=str, nargs='+', help='Whatever goes with the cmd')
#values = args.values
args = parser.parse_args()
cmd = args.cmd[0]
if args.cmd == 'create' and not args.appname or not args.domain:
    logging.error("aegis create requires --appname and --domain")
    sys.exit()

app_name = args.appname[0]
domain = args.domain[0]
stdlib.logw("AEGIS  %s  %s  %s" % (cmd, app_name, domain))
aegis_dir = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
src_dir = os.path.dirname(aegis_dir)


def create(cmd, app_name, domain):
    template_vars = {'app_name': app_name, 'aegis_domain': domain}
    create_dir = os.path.join(src_dir, app_name)
    #stdlib.logw(create_dir, "CREATE DIR")
    #if os.path.exists(create_dir):
    #    logging.error("AEGIS     Sorry that directory exists already. Exiting.")
    #    sys.exit(1)
    if not os.path.exists(create_dir):
        os.mkdir(create_dir)
    create_etc_dir = os.path.join(create_dir, 'etc')
    if not os.path.exists(create_etc_dir):
        os.mkdir(create_etc_dir)
    # Now walk tmpl/etc
    tmpl_dir = os.path.join(aegis_dir, 'aegis', 'tmpl')
    for entry in os.walk(tmpl_dir):
        basedir, subdirs, files = entry
        rebasedir = create_dir + basedir[basedir.find('aegis/tmpl')+10:]
        if rebasedir.endswith('/aegis'):
            rebasedir = rebasedir[:-6] + '/' + app_name
        if not os.path.exists(rebasedir):
            os.mkdir(rebasedir)
        for filename in files:
            filepath = os.path.join(basedir, filename)
            with open(filepath, 'r') as fd:
                # iterate a dictionary of vars and replace vars
                output = fd.read()
                for var, val in template_vars.items():
                    output = output.replace('{{%s}}' % var, val)
                rebase_filename = filename
                if filename == 'aegis.py':
                    rebase_filename = app_name + '.py'
                if filename == 'aegis.conf':
                    rebase_filename = app_name + '.conf'
                if filename == 'aegis_dev.conf':
                    rebase_filename = app_name + '_dev.conf'
                if filename == 'aegis_prod.conf':
                    rebase_filename = app_name + '_prod.conf'
                rebasepath = os.path.join(rebasedir, rebase_filename)
                with open(rebasepath, 'w') as writefd:
                    writefd.write(output)
    print ("GREAT SUCCESS!!")
    # git create and push
    # virtualenv and setup.py
    # epl/epl.py

if __name__ == "__main__":
    if cmd == 'create':
        create(cmd, app_name, domain)
    if cmd == 'apply':
        logging.warn("APPLY ... ETC")
    sys.exit()



#!/usr/bin/env python3

"""
Minimal packaging setup to use command line tools to get started
virtualenv --python=`which python3` virtualenv
. ./virtualenv/bin/activate
python setup.py develop
"""

import os
import setuptools

setuptools.setup (
    name = 'aegis-tools',
    version = '0.0.1',
    description = 'Aegis is a set of battle-tested tools and tricks to help everyone make better software',
    author = "Michael D'Agosta",
    author_email = 'mdagosta@codebug.com',
    url = 'https://bitbucket.org/mdagosta/aegis',
    python_requires='>=3.6',
    packages = ['aegis'],
    entry_points={
        'console_scripts': [
            "aegis = aegis.aegis:main",
        ],
    }
)
